@@ -68,7 +68,7 @@ enum keyword_numeric_config_field {
     KBF_FORWARDCONTROL,
     KBF_OVERRIDEMTU,
     KBF_STRICTCRLPOLICY,
-    KBF_NOCRSEND,
+    KBF_NOCRSEND,     /* 10 */
     KBF_NATTRAVERSAL,
     KBF_KEEPALIVE,
     KBF_PLUTORESTARTONCRASH,
@@ -78,8 +78,8 @@ enum keyword_numeric_config_field {
     KBF_PLUTODEBUG,
     KBF_NHELPERS,
     KBF_OPPOENCRYPT,
-    KBF_DPDDELAY,
-    KBF_DPDTIMEOUT,
+    KBF_DPDDELAY,     /* 20 */
+    KBF_DPDTIMEOUT,   
     KBF_PHASE2,
     KBF_AUTHBY,
     KBF_KEYEXCHANGE,
@@ -88,14 +88,16 @@ enum keyword_numeric_config_field {
     KBF_SALIFETIME,
     KBF_REKEY,
     KBF_REKEYMARGIN,
-    KBF_REKEYFUZZ,
+    KBF_REKEYFUZZ,   /* 30 */
     KBF_COMPRESS,
-    KBF_KEYINGTRIES,
+    KBF_KEYINGTRIES, 
     KBF_ARRIVALCHECK,
     KBF_IKELIFETIME,
     KBF_AGGRMODE,
     KBF_MODECONFIGPULL,
     KBF_FORCEENCAP,
+    KBF_IKEv2,       /* 38 */
+    KBF_CONNADDRFAMILY,
     KBF_MAX         
 };
 
