@@ -745,6 +745,41 @@ struct ikev2_a
 };
 extern struct_desc ikev2_a_desc;
 
+
+/* rfc4306 section 3.6 CERT Payload */
+struct ikev2_cert 
+{
+    u_int8_t  isac_np;	    /* Next payload */
+    u_int8_t  isac_critical;
+    u_int16_t isac_length;	    /* Payload length */
+    u_int8_t  isac_enc;            /* encoding type */
+};
+
+
+/* NOTE: this packet type has a fixed portion that is not a
+ * multiple of 4 octets.  This means that sizeof(struct isakmp_cr)
+ * yields the wrong value for the length.
+ */
+#define IKEV2_CERT_SIZE		5
+extern struct_desc ikev2_certificate_desc;
+
+/* rfc4306 section 3.6 CERTREQ Payload */
+struct ikev2_certreq 
+{
+    u_int8_t  isacertreq_np;	    /* Next payload */
+    u_int8_t  isacertreq_critical;
+    u_int16_t isacertreq_length;	  /* Payload length */
+    u_int8_t  isacertreq_enc;            /* encoding type */
+};
+
+
+/* NOTE: this packet type has a fixed portion that is not a
+ * multiple of 4 octets.  This means that sizeof(struct isakmp_cr)
+ * yields the wrong value for the length.
+ */
+#define IKEV2_CERTREQ_SIZE		5
+extern struct_desc  ikev2_certificate_req_desc;
+
 /* rfc4306, section 3.9, nonce, uses generic header */
 extern struct_desc ikev2_nonce_desc;
 
@@ -798,6 +833,8 @@ union payload {
     struct ikev2_id         v2id;
     struct ikev2_a          v2a;
     struct ikev2_ts         v2ts;
+    struct ikev2_cert       v2cert;
+    struct ikev2_certreq    v2certreq;
 };
 
 
