@@ -12,7 +12,6 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
  *
- * RCSID $Id: addrtot.c,v 1.22.2.2 2007-10-30 21:32:26 paul Exp $
  */
 
 #if defined(__KERNEL__) && defined(__HAVE_ARCH_STRSTR)
@@ -256,6 +255,7 @@ char **dstp;			/* where to put result pointer */
 #ifdef ADDRTOT_MAIN
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -341,8 +341,17 @@ regress()
 
 /*
  * $Log: addrtot.c,v $
- * Revision 1.22.2.2  2007-10-30 21:32:26  paul
+<<<<<<< HEAD:linux/net/ipsec/addrtot.c
+ * Revision 1.24  2005/11/11 06:59:40  mcr
+ * 	try this code to avoid static/extern conflict with newer
+ * 	kernels.
+ *
+ * Revision 1.23  2005/11/11 03:09:53  paul
+ * Fix by Toby for newer kernels that have strstr()
+=======
+ * Revision 1.22.2.2  2007/10/30 21:32:26  paul
  * Added strstr prototype [dhr]
+>>>>>>> 2280ffe...    added prototype for strstr:linux/net/ipsec/addrtot.c
  *
  * Revision 1.22.2.1  2005/11/17 22:30:49  paul
  * pull up strstr fix from head.
@@ -382,9 +391,11 @@ regress()
  * 	if the address type is invalid, then return length of <invalid>
  * 	string!
  *
+<<<<<<< HEAD:linux/net/ipsec/addrtot.c
+=======
  * Revision 1.12  2003/12/30 06:42:48  mcr
  * 	added $Log: addrtot.c,v $
- * 	added Revision 1.22.2.2  2007-10-30 21:32:26  paul
+ * 	added Revision 1.22.2.2  2007/10/30 21:32:26  paul
  * 	added Added strstr prototype [dhr]
  * 	added
  * 	added Revision 1.22.2.1  2005/11/17 22:30:49  paul
@@ -427,5 +438,6 @@ regress()
  * 	added
  *
  *
+>>>>>>> 2280ffe...    added prototype for strstr:linux/net/ipsec/addrtot.c
  */
 
