@@ -14,7 +14,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pf_key.c,v 1.18 2004-04-04 01:50:56 ken Exp $
+ * RCSID $Id: pf_key.c,v 1.18 2004/04/04 01:50:56 ken Exp $
  *
  */
 
@@ -43,15 +43,15 @@
 
 #include <stdint.h>
 #include <openswan.h>
-#include <pfkeyv2.h>
-#include <pfkey.h>
+#include <openswan/pfkeyv2.h>
+#include <openswan/pfkey.h>
 
 char *progname;
 uint32_t pfkey_seq = 0;
 int pfkey_sock;
 
 static void
-Usage(char *progname)
+Usage(void)
 {
 	fprintf(stderr, "%s: Usage: %s [--help]\n"
 		"\tby default listens for AH, ESP, IPIP and IPCOMP\n"
@@ -178,7 +178,7 @@ main(int argc, char *argv[])
 			break;
 			
 		case 'h':
-			Usage(progname);
+			Usage();
 			break;
 		case '0':
 			/* it was a long option with a flag */
@@ -301,7 +301,7 @@ main(int argc, char *argv[])
 	
 /*
  * $Log: pf_key.c,v $
- * Revision 1.18  2004-04-04 01:50:56  ken
+ * Revision 1.18  2004/04/04 01:50:56  ken
  * Use openswan includes
  *
  * Revision 1.17  2003/11/07 01:36:55  ken
