@@ -42,6 +42,7 @@
 #include <security/pam_appl.h>
 #endif
 #include "connections.h"	/* needs id.h */
+#include "cookie.h"
 #include "state.h"
 #include "packet.h"
 #include "md5.h"
@@ -300,13 +301,15 @@ process_v2_packet(struct msg_digest **mdp)
 
 	md->role = RESPONDER;
 
-	if(rcookiezero && md->msgid_received==MAINMODE_MSGID) {
-	    /* this is the first message, no state expected */
-	    st = NULL; 
-	} else {
-	    st = find_state_ikev2_parent(md->hdr.isa_icookie
-					 , md->hdr.isa_rcookie);
+	st = find_state_ikev2_parent(md->hdr.isa_icookie
+				     , md->hdr.isa_rcookie);
 
+	if(st == NULL) {
+	    /* first time for this cookie, it's a new state! */
+	    st = find_state_ikev2_parent_init(md->hdr.isa_icookie);
+	}
+
+	if(st) {
 	    if(st->st_msgid_lastrecv >  md->msgid_received){
 		/* this is an OLD retransmit. we can't do anything */
 		openswan_log("received too old retransmit: %u < %u"
@@ -355,27 +358,31 @@ process_v2_packet(struct msg_digest **mdp)
 		 */
 		st = find_state_ikev2_parent(md->hdr.isa_icookie
 					     , md->hdr.isa_rcookie);
-		if(st) {
-		    /*
-		     * then there is something wrong with the msgid, so
-		     * maybe they retransmitted for some reason. 
-		     * Check if it's an old packet being returned, and
-		     * if so, drop it.
-		     * NOTE: in_struct() changed the byte order.
-		     */
-		    if(md->msgid_received <= st->st_msgid_lastack) {
-			/* it's fine, it's just a retransmit */
-			DBG(DBG_CONTROL, DBG_log("responding peer retransmitted msgid %u"
-						 , md->msgid_received));
-			return;
-		    }
-		    openswan_log("last msgid ack is %u, received: %u"
-				 , st->st_msgid_lastack
-				 , md->msgid_received);
-		    return;
-		}
 	    }
 	}
+
+	if(st) {
+	    /*
+	     * then there is something wrong with the msgid, so
+	     * maybe they retransmitted for some reason. 
+	     * Check if it's an old packet being returned, and
+	     * if so, drop it.
+	     * NOTE: in_struct() changed the byte order.
+	     */
+	    if(st->st_msgid_lastack != INVALID_MSGID
+	       && md->msgid_received <= st->st_msgid_lastack) {
+		/* it's fine, it's just a retransmit */
+		DBG(DBG_CONTROL, DBG_log("responding peer retransmitted msgid %u"
+					 , md->msgid_received));
+		return;
+	    }
+#if 0
+	    openswan_log("last msgid ack is %u, received: %u"
+			 , st->st_msgid_lastack
+			 , md->msgid_received);
+	    return;
+#endif
+	}
     }
 	
     ix = md->hdr.isa_xchg;
@@ -552,7 +559,6 @@ void ikev2_log_parentSA(struct state *st)
     }
 }
 
-
 void
 send_v2_notification_from_state(struct state *st, enum state_kind state,
 				u_int16_t type)
@@ -562,12 +568,16 @@ send_v2_notification_from_state(struct state *st, enum state_kind state,
     if (state == STATE_UNDEFINED)
 	state = st->st_state;
 
-    openswan_log("Sending notification %u", type);
+    send_v2_notification(st, type, NULL, 0
+			 , st->st_icookie, st->st_rcookie);
 }
 
 void
 send_v2_notification_from_md(struct msg_digest *md UNUSED, u_int16_t type)
 {
+    struct state st;
+    struct connection cnx;
+
     /**
      * Create a dummy state to be able to use send_packet in
      * send_notification
@@ -577,13 +587,6 @@ send_v2_notification_from_md(struct msg_digest *md UNUSED, u_int16_t type)
      *   st_connection->that.host_port
      *   st_connection->interface
      */
-    openswan_log("Sending notification %u", type);
-
-#if 0
-    struct state st;
-    struct connection cnx;
-
-
     passert(md);
 
     memset(&st, 0, sizeof(st));
@@ -596,24 +599,31 @@ send_v2_notification_from_md(struct msg_digest *md UNUSED, u_int16_t type)
     cnx.interface = md->iface;
     st.st_interface = md->iface;
 
-    send_notification(&st, type, NULL, 0,
-	md->hdr.isa_icookie, md->hdr.isa_rcookie, NULL, 0, PROTO_ISAKMP);
-#endif
+    send_v2_notification(&st, type, NULL, 0,
+			 md->hdr.isa_icookie, md->hdr.isa_rcookie);
 }
 
 void ikev2_update_counters(struct msg_digest *md)
 {
+    struct state *pst= md->pst;
     struct state *st = md->st;
+
+    if(pst==NULL) {
+	if(st->st_clonedfrom != 0) {
+	    pst = state_with_serialno(st->st_clonedfrom);
+	}
+	if(pst == NULL) pst = st;
+    }
     
     switch(md->role) {
     case INITIATOR:
 	/* update lastuse values */
-	st->st_msgid_lastack = md->msgid_received;
-	st->st_msgid_nextuse = st->st_msgid_lastack+1;
+	pst->st_msgid_lastack = md->msgid_received;
+	pst->st_msgid_nextuse = pst->st_msgid_lastack+1;
 	break;
 	
     case RESPONDER:
-	st->st_msgid_lastrecv= md->msgid_received;
+	pst->st_msgid_lastrecv= md->msgid_received;
 	break;
     }
 }
@@ -683,7 +693,6 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	       , sadetails);
     }
 
-
     /* free previous transmit packet */
     freeanychunk(st->st_tpacket);
 
