@@ -135,6 +135,11 @@ extern stf_status ikev2_child_sa_respond(struct msg_digest *md
 extern struct traffic_selector ikev2_subnettots(struct end *e);
 extern void ikev2_update_counters(struct msg_digest *md);
 
+extern void send_v2_notification(struct state *p1st, u_int16_t type
+				 , struct state *encst, msgid_t msgid
+				 , u_char *icookie
+				 , u_char *rcookie);
+
 extern bool doi_send_ikev2_cert_thinking( struct state *st);
 
 extern stf_status ikev2_send_cert( struct state *st
@@ -142,3 +147,4 @@ extern stf_status ikev2_send_cert( struct state *st
 				   , enum phase1_role role
 				   , unsigned int np
 				   , pb_stream *outpbs);
+
