@@ -1,6 +1,8 @@
 /* 
  * IKEv2 parent SA creation routines
  * Copyright (C) 2007  Michael Richardson <mcr@xelerance.com>
+ * Copyright (C) 2008  Paul Wouters <paul@xelerance.com>
+ * Copyright (C) 2008  Antony Antony <antony@xelerance.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -95,6 +97,7 @@ ikev2parent_outI1(int whack_sock
 
 
     /* set up new state */
+    get_cookie(TRUE, st->st_icookie, COOKIE_SIZE, &c->spd.that.host_addr);
     initialize_new_state(st, c, policy, try, whack_sock, importance);
     st->st_ikev2 = TRUE;
     st->st_state = STATE_PARENT_I1;
@@ -168,7 +171,7 @@ ikev2parent_outI1(int whack_sock
 	}
     }
     if(groupnum == 0) {
-	groupnum = OAKLEY_GROUP_MODP2048;
+	groupnum = OAKLEY_GROUP_MODP1536;
     }
     st->st_oakley.group=lookup_group(groupnum); 
 
@@ -448,6 +451,7 @@ stf_status ikev2parent_inI1outR1(struct msg_digest *md)
     if(!st) {
 	st = new_state();
 	/* set up new state */
+	memcpy(st->st_icookie, md->hdr.isa_icookie, COOKIE_SIZE);
 	initialize_new_state(st, c, policy, 0, NULL_FD, pcim_stranger_crypto);
 	st->st_ikev2 = TRUE;
 	st->st_state = STATE_PARENT_R1;
@@ -554,10 +558,9 @@ ikev2_parent_inI1outR1_tail(struct pluto_crypto_req_cont *pcrc
     pb_stream *keyex_pbs;
     int    numvidtosend=1;
 
-    unhash_state(st);
+    /* note that we don't update the state here yet */
     memcpy(st->st_icookie, md->hdr.isa_icookie, COOKIE_SIZE);
     get_cookie(FALSE, st->st_rcookie, COOKIE_SIZE, &md->sender);
-    insert_state(st);	
 
     /* record first packet for later checking of signature */
     clonetochunk(st->st_firstpacket_him, md->message_pbs.start
@@ -772,7 +775,7 @@ static void ikev2_padup_pre_encrypt(struct msg_digest *md
 				    , pb_stream *e_pbs_cipher)
 {
     struct state *st = md->st;
-    struct state *pst = st;
+    struct state *pst= st;
     
     if(st->st_clonedfrom != 0) {
 	pst = state_with_serialno(st->st_clonedfrom);
@@ -1001,7 +1004,6 @@ static stf_status ikev2_send_auth(struct connection *c
 	    return STF_FATAL;
 	
     } else if(c->policy & POLICY_PSK) {
-	/* todo */
 	if(!ikev2_calculate_psk_auth(pst, role, idhash_out, &a_pbs))
 	return STF_FAIL;
     } 
@@ -1040,6 +1042,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
     st->st_msgid = htonl(pst->st_msgid_nextuse);
     insert_state(st);
     md->st = st;
+    md->pst= pst;
 
     /* need to force parent state to I2 */
     pst->st_state = STATE_PARENT_I2;
@@ -1097,6 +1100,7 @@ ikev2_parent_inR1outI2_tail(struct pluto_crypto_req_cont *pcrc
 	unsigned char *id_start;
 	unsigned int   id_len;
 
+	/* XXX probably should be prf_hasher, not integ_hasher */
 	hmac_init_chunk(&id_ctx, pst->st_oakley.integ_hasher, pst->st_skey_pi);
 	build_id_payload((struct isakmp_ipsec_id *)&r_id, &id_b, &c->spd.this);
 	r_id.isai_critical = ISAKMP_PAYLOAD_CRITICAL;
@@ -1329,6 +1333,11 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	if(ret != STF_OK) return ret;
     }
 
+    /* if it decrypted okay, then things are good, this packet is
+     * well received, and we should change state.
+     */
+    rehash_state(st);
+
     if(!ikev2_decode_peer_id(md, RESPONDER)) {
 	return STF_FAIL + INVALID_ID_INFORMATION;
     }
@@ -1351,7 +1360,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 
     /* process CERT payload */
     {
-    if(md->chain[ISAKMP_NEXT_v2CERT])
+	if(md->chain[ISAKMP_NEXT_v2CERT])
 	{
 	    /* should we check if we should accept a cert payload ?
 	     *  has_preloaded_public_key(st)
@@ -1361,7 +1370,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 		, DBG_log("has a v2_CERT payload going to process it "));	  
 	    ikev2_decode_cert(md); 
 	}
-     }
+    }
 
     /* process CERTREQ payload */
     if(md->chain[ISAKMP_NEXT_v2CERTREQ]) 
@@ -1402,7 +1411,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 						    , idhash_in
 						    , &md->chain[ISAKMP_NEXT_v2AUTH]->pbs);
 	if(authstat != STF_OK) {
-	    openswan_log("PSK authentication failed");
+	    openswan_log("PSK authentication failed AUTH mismatch!");
 	    SEND_NOTIFICATION(AUTHENTICATION_FAILED);
 	    return STF_FAIL;
 	}
@@ -1430,6 +1439,7 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	struct ikev2_generic e;
 	pb_stream      e_pbs, e_pbs_cipher;
 	stf_status     ret;
+	bool send_cert = FALSE;
 
 	/* HDR out */
 	{
@@ -1467,7 +1477,9 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	e_pbs_cipher.cur = e_pbs.cur;
 	encstart = e_pbs_cipher.cur;
 	
-         bool send_cert = FALSE;
+	/* decide to send CERT payload before we generate IDr */
+	send_cert = doi_send_ikev2_cert_thinking(st);
+	    
 	/* send out the IDr payload */
 	{
 	    struct ikev2_id r_id;
@@ -1482,14 +1494,12 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	    build_id_payload((struct isakmp_ipsec_id *)&r_id, &id_b,
 			     &c->spd.this);
 	    r_id.isai_critical = ISAKMP_PAYLOAD_CRITICAL;
-           {  /* decide to send CERT payload */
-	    send_cert = doi_send_ikev2_cert_thinking(st);
-	    
+
 	    if(send_cert) 
 		r_id.isai_np = ISAKMP_NEXT_v2CERT;
 	    else  
 		r_id.isai_np = ISAKMP_NEXT_v2AUTH; 
-	    }
+
 	    id_start = e_pbs_cipher.cur;
 	    
 	    if (!out_struct(&r_id
@@ -1512,17 +1522,15 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	}
 
 	DBG(DBG_CONTROLMORE
-	    , DBG_log("assembeled IDr payload"));
-	
+	    , DBG_log("assembled IDr payload -- CERT next"));	  
+
 	/* send CERT payload RFC 4306 3.6, 1.2:([CERT,] ) */
-	{
-	    if(send_cert){
-		stf_status certstat = ikev2_send_cert( st, md
-						       , RESPONDER
-						       ,ISAKMP_NEXT_v2AUTH
-						       , &e_pbs_cipher);
-	    	if(certstat != STF_OK) return certstat;
-	    }
+	if(send_cert) {
+	    stf_status certstat = ikev2_send_cert(st, md
+						  , RESPONDER
+						  , ISAKMP_NEXT_v2AUTH
+						  , &e_pbs_cipher);
+	    if(certstat != STF_OK) return certstat;
     	} 
 
 	/* authentication good, see if there is a child SA available */
@@ -1536,9 +1544,10 @@ ikev2_parent_inI2outR2_tail(struct pluto_crypto_req_cont *pcrc
 	} else {
 	    np = ISAKMP_NEXT_v2SA;
 	}
-	
+
 	DBG(DBG_CONTROLMORE
-	    , DBG_log("going to assmemble AUTH payload"));	  
+	    , DBG_log("going to assemble AUTH payload"));	  
+
 	/* now send AUTH payload */
 	{
 	    stf_status authstat = ikev2_send_auth(c, st
@@ -1649,18 +1658,16 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
 	idhash_in = alloca(pst->st_oakley.integ_hasher->hash_digest_len);
 	hmac_final(idhash_in, &id_ctx);
     }
-    {
-    if(md->chain[ISAKMP_NEXT_v2CERT])
-	{
-	    /* should we check if we should accept a cert payload ?
-	     *  has_preloaded_public_key(st)
-	     */ 
-	    /* in v1 code it is  decode_cert(struct msg_digest *md) */
-	    DBG(DBG_CONTROLMORE
-		, DBG_log("has a v2_CERT payload going to decode it"));	  
-	    ikev2_decode_cert(md); 
-	}
-     }
+
+    if(md->chain[ISAKMP_NEXT_v2CERT]) {
+	/* should we check if we should accept a cert payload ?
+	 *  has_preloaded_public_key(st)
+	 */ 
+	/* in v1 code it is  decode_cert(struct msg_digest *md) */
+	DBG(DBG_CONTROLMORE
+	    , DBG_log("has a v2_CERT payload going to decode it"));	  
+	ikev2_decode_cert(md); 
+    }
 
     /* process AUTH payload */
     if(!md->chain[ISAKMP_NEXT_v2AUTH]) {
@@ -1745,6 +1752,60 @@ stf_status ikev2parent_inR2(struct msg_digest *md)
     
 }
 
+/*
+ *
+ ***************************************************************
+ *                       NOTIFICATION_OUT                  *****
+ ***************************************************************
+ *
+ */
+
+void
+send_v2_notification(struct state *p1st, u_int16_t type
+		     , struct state *encst
+		     , msgid_t msgid UNUSED
+		     , u_char *icookie UNUSED
+		     , u_char *rcookie UNUSED)
+{
+    u_char buffer[1024];
+    pb_stream pbs;
+
+    /*
+     * no complex timers like in IKEv1, because notifications are
+     * only sent in the responder, in response to a request.
+     */
+
+    openswan_log("sending %snotification %s to %s:%u"
+		 , encst ? "encrypted " : ""
+		 , enum_name(&ipsec_notification_names, type)
+		 , ip_str(&p1st->st_remoteaddr)
+		 , p1st->st_remoteport);
+
+    memset(buffer, 0, sizeof(buffer));
+    init_pbs(&pbs, buffer, sizeof(buffer), "notification msg");
+
+#if 0
+    /* HDR* */
+    {
+	hdr.isa_version = ISAKMP_MAJOR_VERSION << ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION;
+	hdr.isa_np = encst ? ISAKMP_NEXT_HASH : ISAKMP_NEXT_N;
+	hdr.isa_xchg = ISAKMP_XCHG_INFO;
+	hdr.isa_msgid = msgid;
+	hdr.isa_flags = encst ? ISAKMP_FLAG_ENCRYPTION : 0;
+	if (icookie)
+	    memcpy(hdr.isa_icookie, icookie, COOKIE_SIZE);
+	if (rcookie)
+	    memcpy(hdr.isa_rcookie, rcookie, COOKIE_SIZE);
+	if (!out_struct(&hdr, &isakmp_hdr_desc, &pbs, &r_hdr_pbs))
+	    impossible();
+    }
+#endif
+
+    /* XXX */
+    return;
+}
+		     
+
 /*
  *
  ***************************************************************
