@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char spi_c_version[] = "RCSID $Id: spi.c,v 1.112.2.3 2006-02-15 04:36:36 paul Exp $";
+char spi_c_version[] = "RCSID $Id: spi.c,v 1.114 2005/08/18 14:04:40 ken Exp $";
 
 #include <asm/types.h>
 #include <sys/types.h>
@@ -44,10 +44,10 @@ char spi_c_version[] = "RCSID $Id: spi.c,v 1.112.2.3 2006-02-15 04:36:36 paul Ex
 #if 0
 #include <linux/autoconf.h>    /* CONFIG_IPSEC_PFKEYv2 */
 #endif
-     #include <signal.h>
-     #include <sys/socket.h>
-     #include <pfkeyv2.h>
-     #include <pfkey.h>
+#include <signal.h>
+#include <sys/socket.h>
+#include <openswan/pfkeyv2.h>
+#include <openswan/pfkey.h>
 
 #include "openswan/radij.h"
 #include "openswan/ipsec_encap.h"
@@ -61,6 +61,7 @@ char spi_c_version[] = "RCSID $Id: spi.c,v 1.112.2.3 2006-02-15 04:36:36 paul Ex
 #include "oswlog.h"
 #include "alg_info.h"
 #include "kernel_alg.h"
+#include "pfkey_help.h"
 
 struct encap_msghdr *em;
 
@@ -76,7 +77,7 @@ char *command;
 extern char *optarg;
 extern int optind, opterr, optopt;
 char scratch[2];
-char *iv = NULL, *enckey = NULL, *authkey = NULL;
+unsigned char *iv = NULL, *enckey = NULL, *authkey = NULL;
 size_t ivlen = 0, enckeylen = 0, authkeylen = 0;
 ip_address edst, dst, src;
 int address_family = 0;
@@ -157,11 +158,11 @@ usage(char *s, FILE *f)
 }
 
 int
-parse_life_options(uint32_t life[life_maxsever][life_maxtype],
+parse_life_options(u_int32_t life[life_maxsever][life_maxtype],
 		   char *life_opt[life_maxsever][life_maxtype],
-		   char *optarg)
+		   char *myoptarg)
 {
-	char *optargp = optarg;
+	char *optargp = myoptarg;
 	char *endptr;
 	
 	do {
@@ -285,16 +286,17 @@ parse_life_options(uint32_t life[life_maxsever][life_maxtype],
 				"%s: Invalid character='%c' at offset %d in lifetime option parameter: '%s', parameter string is %d characters long, %d valid value characters found.\n",
 				progname,
 				*endptr,
-				(int)(endptr - optarg),
-				optarg,
-				(int)strlen(optarg),
+				(int)(endptr - myoptarg),
+				myoptarg,
+				(int)strlen(myoptarg),
 				(int)(strcspn(optargp, ", ") - 1));
 			return(1);
 		}
 		life_opt[life_severity][life_type] = optargt;
 		if(debug) {
-			fprintf(stdout, "%s lifetime %s set to %d.\n",
-				progname, optargt, life[life_severity][life_type]);
+			fprintf(stdout, "%s lifetime %s set to %lu.\n",
+				progname, optargt,
+				(unsigned long)life[life_severity][life_type]);
 		}
 		optargp=endptr+1;
 	} while(*endptr==',' || isspace(*endptr));
@@ -389,7 +391,7 @@ static struct option const longopts[] =
 	{0, 0, 0, 0}
 };
 
-#ifdef NAT_TRAVERSAL
+
 static bool
 pfkey_build(int error
 	    , const char *description
@@ -408,7 +410,7 @@ pfkey_build(int error
 	return FALSE;
     }
 }
-#endif
+
 int decode_esp(char *algname)
 {
   int esp_alg;
@@ -860,7 +862,7 @@ main(int argc, char *argv[])
 				exit(1);
 			}
 			memset(authkey, 0, authkeylen);
-			authkeylen = atodata(optarg, 0, authkey, authkeylen);
+			authkeylen = atodata(optarg, 0, (char *)authkey, authkeylen);
 			akey_opt = optarg;
 			break;
 		case 'E':
@@ -888,7 +890,7 @@ main(int argc, char *argv[])
 				exit(1);
 			}
 			memset(enckey, 0, enckeylen);
-			enckeylen = atodata(optarg, 0, enckey, enckeylen);
+			enckeylen = atodata(optarg, 0, (char *)enckey, enckeylen);
 			ekey_opt = optarg;
 			break;
 		case 'w':
@@ -929,7 +931,7 @@ main(int argc, char *argv[])
 				exit(1);
 			}
 			memset(iv, 0, ivlen);
-			ivlen = atodata(optarg, 0, iv, ivlen);
+			ivlen = atodata(optarg, 0, (char *)iv, ivlen);
 			iv_opt = optarg;
 			break;
 		case 'D':
@@ -1188,54 +1190,12 @@ main(int argc, char *argv[])
 			progname);
 	}
 
-	if((pfkey_sock = socket(PF_KEY, SOCK_RAW, PF_KEY_V2) ) < 0) {
-		fprintf(stderr, "%s: Trouble opening PF_KEY family socket with error: ",
-			progname);
-		switch(errno) {
-		case ENOENT:
-			fprintf(stderr, "device does not exist.  See FreeS/WAN installation procedure.\n");
-			break;
-		case EACCES:
-			fprintf(stderr, "access denied.  ");
-			if(getuid() == 0) {
-				fprintf(stderr, "Check permissions.  Should be 600.\n");
-			} else {
-				fprintf(stderr, "You must be root to open this file.\n");
-			}
-			break;
-		case EUNATCH:
-			fprintf(stderr, "Netlink not enabled OR KLIPS not loaded.\n");
-			break;
-		case ENODEV:
-			fprintf(stderr, "KLIPS not loaded or enabled.\n");
-			break;
-		case EBUSY:
-			fprintf(stderr, "KLIPS is busy.  Most likely a serious internal error occured in a previous command.  Please report as much detail as possible to development team.\n");
-			break;
-		case EINVAL:
-			fprintf(stderr, "Invalid argument, KLIPS not loaded or check kernel log messages for specifics.\n");
-			break;
-		case ENOBUFS:
-			fprintf(stderr, "No kernel memory to allocate SA.\n");
-			break;
-		case ESOCKTNOSUPPORT:
-			fprintf(stderr, "Algorithm support not available in the kernel.  Please compile in support.\n");
-			break;
-		case EEXIST:
-			fprintf(stderr, "SA already in use.  Delete old one first.\n");
-			break;
-		case ENXIO:
-			fprintf(stderr, "SA does not exist.  Cannot delete.\n");
-			break;
-		case EAFNOSUPPORT:
-			fprintf(stderr, "KLIPS not loaded or enabled.\n");
-			break;
-		default:
-			fprintf(stderr, "Unknown file open error %d.  Please report as much detail as possible to development team.\n", errno);
-		}
-		exit(1);
+	pfkey_sock = pfkey_open_sock_with_error();
+	if(pfkey_sock < 0) {
+	    exit(1);
 	}
 
+
 #ifdef MANUAL_IS_NOT_ABLE_TO_NEGOTIATE
 	/* for registering SA types that can be negotiated */
 	if(pfkey_register(SADB_SATYPE_AH) != 0) {
@@ -1317,7 +1277,7 @@ main(int argc, char *argv[])
 					   SADB_EXT_SA,
 					   htonl(spi), /* in network order */
 					   replay_window,
-					   SADB_SASTATE_MATURE,
+					   K_SADB_SASTATE_MATURE,
 					   authalg,
 					   encryptalg,
 					   0))) {
@@ -1645,7 +1605,7 @@ main(int argc, char *argv[])
 			fprintf(stderr, "SA already in use.  Delete old one first.\n");
 			break;
 		case ENOENT:
-			fprintf(stderr, "device does not exist.  See FreeS/WAN installation procedure.\n");
+			fprintf(stderr, "device does not exist.  See Openswan installation procedure.\n");
 			break;
 		case ENXIO:
 		case ESRCH:
@@ -1701,8 +1661,8 @@ main(int argc, char *argv[])
 		unsigned char pfkey_buf[PFKEYv2_MAX_MSGSIZE];
 		
 		while((readlen = read(pfkey_sock, pfkey_buf, sizeof(pfkey_buf))) > 0) {
-			struct sadb_ext *extensions[SADB_EXT_MAX + 1];
-			pfkey_extensions_init(extensions);
+			struct sadb_ext *extensions2[SADB_EXT_MAX + 1];
+			pfkey_extensions_init(extensions2);
 			pfkey_msg = (struct sadb_msg *)pfkey_buf;
 			
 			/* first, see if we got enough for an sadb_msg */
@@ -1741,7 +1701,7 @@ main(int argc, char *argv[])
 				continue;
 			}
 			
-			if (pfkey_msg_parse(pfkey_msg, NULL, extensions, EXT_BITS_OUT)) {
+			if (pfkey_msg_parse(pfkey_msg, NULL, extensions2, EXT_BITS_OUT)) {
 				if(debug) {
 					printf("%s: unparseable PF_KEY message.\n",
 					       progname);
@@ -1757,8 +1717,8 @@ main(int argc, char *argv[])
 				if(saref) {
 					printf("%s: saref=%d\n",
 					       progname,
-					       (extensions[SADB_EXT_SA] != NULL)
-					       ? ((struct sadb_sa*)(extensions[SADB_EXT_SA]))->sadb_x_sa_ref
+					       (extensions2[SADB_EXT_SA] != NULL)
+					       ? ((struct sadb_sa*)(extensions2[SADB_EXT_SA]))->sadb_x_sa_ref
 					       : IPSEC_SAREF_NULL);
 				}
 				break;
@@ -1779,18 +1739,14 @@ void exit_tool(int x)
 
 /*
  * $Log: spi.c,v $
- * Revision 1.112.2.3  2006-02-15 04:36:36  paul
- * Added #ifdef NAT_TRAVERSAL for pfkey_build. The 'unused' warning caused
- * a compile failure when compiling without NAT_TRAVERSAL.
- *
- * Revision 1.112.2.2  2005/08/18 14:04:51  ken
+ * Revision 1.114  2005/08/18 14:04:40  ken
  * Patch from mt@suse.de to avoid GCC warnings with system() calls
  *
- * Revision 1.112.2.1  2005/08/12 16:30:32  ken
- * Make tree PG13
+ * Revision 1.113  2005/08/05 08:42:45  mcr
+ * 	signed/unsigned adjustments.
  *
  * Revision 1.112  2005/07/08 02:55:55  paul
- * fix gcc warning
+ * fuck vault
  *
  * Revision 1.111  2005/05/12 03:08:23  mcr
  * 	do not mess with keysize for 3des/des.
