@@ -443,6 +443,8 @@ recordresults() {
 	copybadresults=true
     fi
 
+    ECHO=${ECHO-echo}
+
     export REGRESSRESULTS
     roguekill $REPORT_NAME
 
@@ -483,7 +485,7 @@ recordresults() {
 	    *)	success=false;;
 	esac
 
-	echo "Recording "'"'"$success: $status"'"'" to $REGRESSRESULTS/$REPORT_NAME/status"
+	${ECHO} "Recording "'"'"$success: $status"'"'" to $REGRESSRESULTS/$REPORT_NAME/status"
 	echo "$success: $status" >$REGRESSRESULTS/$REPORT_NAME/status
 	echo console=$console >>$REGRESSRESULTS/$REPORT_NAME/status
 	echo packet=$packet   >>$REGRESSRESULTS/$REPORT_NAME/status
@@ -563,6 +565,9 @@ pcap_filter() {
 	FILTER="$FILTER | sed -f $FIXUPDIR/tcpdump-three-eight.sed"
     fi
 
+    PATH=$PATH:../../klips/fixups
+    export PATH
+
     if [ -n "${OUTPUT-}" ]
     then
 	rm -f OUTPUT${KLIPS_MODULE}/${OUTPUT}.txt
@@ -1104,15 +1109,9 @@ complibtest() {
     elif [ -f ${OPENSWANSRCDIR}/lib/libopenswan/$testsrc ]
     then
         FILE=${OPENSWANSRCDIR}/lib/libopenswan/$testsrc
-    elif [ -f ${OPENSWANSRCDIR}/linux/net/klips/$testsrc ]
-    then
-        FILE=${OPENSWANSRCDIR}/linux/net/klips/$testsrc
     elif [ -f ${OPENSWANSRCDIR}/linux/lib/libopenswan/$testsrc ]
     then
         FILE=${OPENSWANSRCDIR}/linux/lib/libopenswan/$testsrc
-    elif [ -f ${OPENSWANSRCDIR}/linux/lib/libfreeswan/$testsrc ]
-    then
-        FILE=${OPENSWANSRCDIR}/linux/lib/libfreeswan/$testsrc
     elif [ -f ${OPENSWANSRCDIR}/linux/net/ipsec/$testsrc ]
     then
         FILE=${OPENSWANSRCDIR}/linux/net/ipsec/$testsrc
@@ -1167,7 +1166,6 @@ libtest() {
 	    ${ECHO} "   "Running $testobj
 	    ( ulimit -c unlimited; cd lib-$testobj && eval ../$testobj ${UNITTESTARGS} >OUTPUT${KLIPS_MODULE}/$testobj.txt 2>&1 )
 
-
 	    stat=$?
 	    ${ECHO} "   "Exit code $stat
 	    if [ $stat -gt 128 ]
@@ -1188,7 +1186,7 @@ libtest() {
             fi
     fi
 
-    TEST_PURPOSE=regress  UML_BRAND=0 recordresults lib-$testobj "$testexpect" $stat lib-$testobj
+    TEST_PURPOSE=regress  UML_BRAND=0 recordresults lib-$testobj "$testexpect" "$stat" lib-$testobj
 }
 
 # test entry point:
@@ -1210,30 +1208,30 @@ multilibtest() {
 	    echo "file ../$testobj" >lib-$testobj/.gdbinit
 	    echo "set args "${UNITTESTARGS} >>lib-$testobj/.gdbinit
 
-	    echo Running $testobj ${UNITTESTARGS}
+	    ${ECHO} Multilib running lib-$testobj/testlist.sh for $testobj ${UNITTESTARGS}
 	    ( ulimit -c unlimited; cd lib-$testobj && ./testlist.sh >OUTPUT${KLIPS_MODULE}/$testobj.txt 2>&1 )
 
 	    stat=$?
-	    echo Exit code $stat
+	    ${ECHO} Exit code $stat
 	    if [ $stat -gt 128 ]
 	    then
 		stat="$stat core"
 	    else
 		if [ -r OUTPUT.$testobj.txt ]
 		then
-		    if diff -N -u -w -b -B lib-$testobj/OUTPUT${KLIPS_MODULE}/$testobj.txt OUTPUT.$testobj.txt > lib-$testobj/OUTPUT${KLIPS_MODULE}/$testobj.output.diff
+		    if diff -N -u -w -b -B OUTPUT.$testobj.txt lib-$testobj/OUTPUT${KLIPS_MODULE}/$testobj.txt > lib-$testobj/OUTPUT${KLIPS_MODULE}/$testobj.output.diff
 		    then
-			echo "output matched"
+			${ECHO} "output matched"
 			stat="0"
 		    else
-			echo "output differed"
+			echo "$testobj: output differed"
 			stat="1"
 		    fi
 		fi
             fi
     fi
 
-    TEST_PURPOSE=regress  UML_BRAND=0 recordresults lib-$testobj "$testexpect" $stat lib-$testobj
+    TEST_PURPOSE=regress  UML_BRAND=0 recordresults lib-$testobj "$testexpect" "$stat" lib-$testobj false
 }
 
 
@@ -1854,8 +1852,15 @@ do_unittest() {
 	if [ -d $program ]; then (cd $program && make programs checkprograms ); fi
      done)
 
+    echo "BUILDING TEST CASE"
     # if there is a makefile, run it and bail if fails
-    [ -f Makefile ] && make checkprograms
+    if [ -f Makefile ]; then
+	if make checkprograms; then
+	    :
+	else
+	    exit 1;
+	fi
+    fi
 
     # make sure we get all core dumps!
     ulimit -c unlimited
@@ -1863,11 +1868,11 @@ do_unittest() {
 
     OUTDIR=${OBJDIRTOP}/testing/${TESTSUBDIR}/${TESTNAME}
     mkdir -p ${OUTDIR}
-    ln -f -s ${OUTDIR} OUTPUT
+    rm -f OUTPUT; ln -f -s ${OUTDIR} OUTPUT
 
     echo "RUNNING $TESTSCRIPT"
     ./$TESTSCRIPT >${OUTDIR}/console.txt
-    echo "DONE $TESTSCRIPT"
+    echo "DONE    $TESTSCRIPT"
 
     stat=$?
     echo Exit code $stat
